"""
This module contains a method to automatically detect the mimetype of
files. Currently, type inference is solely based on the extension of a
file. The class %%vhistfile.VhistFile uses this module to automatically
derive mimetypes for workflow files without a user-specified filetype.
"""

import os
import mimetypes
import iomonitor
import __builtin__

def getMimeType(filename):
    """ Tries to identify the MIME type of the given file, based on the
    file's extension.
    
    ``filename`` is the name of the file, for which you want to identify
    the MIME type.
    
    This function is called automatically every type you create a
    %%vhistfile.VhistFile instance and do not explicitly specify a MIME
    type.
    """
    
    replaceOpen = __builtin__.open
    try:
        __builtin__.open = iomonitor.originalOpen
        mimetypes.init()
    finally:
        __builtin__.open = replaceOpen
    
    # get file extension without the leading dot "."
    (basefilename, ext) = os.path.splitext(filename)
    
    if ext in mimetypes.types_map:
        return mimetypes.types_map[ext]
    else:
        return None


