
import __builtin__
import os

import vhistfile

"""
TODO:
    - monitor file, open, codecs.open etc
    - monitor shutil and os routines (move, rename etc)
    - autodetect vhist files for input files

DONE:
    - works with open
    - works with gzip.open
    - works with PIL (only tested with one png image, reading)
    - probably works with nibabel

NOT DONE:
    - file (subclassing file breaks isinstance(x, file) calls
    - codecs.open
    - io.open
    - check submodules
"""

originalOpen = open
#originalFile = file


def dontRecordFileAccess(func, *args, **kargs):
    newOpen = __builtin__.open
    try:
        __builtin__.open = originalOpen
        return func(*args, **kargs)
    finally:
        __builtin__.open = newOpen


class IOMonitor(object):
    def __init__(self, section, maxSizeBytes = float("inf"),
            firstVhistAsRoot=True):
        self.oldOpen = None
        self.vhistSection = section
        self.maxSize = maxSizeBytes
        
        self.rootFilename = None
        self.vhistOutfiles = []
        
        self.infiles = []
        self.outfiles = []
    
    def __enter__(self):
        self.oldOpen = __builtin__.open
        #self.oldFile = __builtin__.file
        __builtin__.open = self.genOpen()
        #__builtin__.file = self.genFile()
        return self
    
    def __exit__(self, type, value, traceback):
        __builtin__.open = self.oldOpen
        #__builtin__.file = self.oldFile
        self.addFilesToSection()
    
    def addFilesToSection(self):
        foundAVhistFile = False
        for infile in self.infiles:
            vfile = infile + ".vhist"
            if os.path.isfile(infile):
                vf = vhistfile.InFile(infile)
                vf.embed = os.path.getsize(infile) < self.maxSize
                self.vhistSection.files.append(vf)
            if os.path.isfile(vfile):
                if foundAVhistFile:
                    vf = vhistfile.InFile(vfile)
                    vf.embed = os.path.getsize(vfile) < self.maxSize
                    self.vhistSection.files.append(vf)
                else:
                    self.rootFilename = vfile
                    foundAVhistFile = True
        
        for outfile in self.outfiles:
            vfile = outfile + ".vhist"
            if os.path.isfile(outfile):
                vf = vhistfile.OutFile(outfile)
                vf.embed = os.path.getsize(outfile) < self.maxSize
                self.vhistSection.files.append(vf)
            self.vhistOutfiles.append(vfile)
    
    def genOpen(self):
        def monitoredOpen(name, mode="r", buffering=-1):
            #print name, mode
            if "r" in mode:
                self.infiles.append(os.path.abspath(name))
            elif "w" in mode or "a" in mode:
                self.outfiles.append(os.path.abspath(name))
            return self.oldOpen(name, mode, buffering)
        return monitoredOpen
    
    def genFile(self):
        withBlock = self
        class MonitoredFile(withBlock.oldFile):
            def __init__(self, name, mode="r", buffering=-1):
                #print name, mode
                if "r" in mode:
                    withBlock.infiles.append(os.path.abspath(name))
                elif "w" in mode or "a" in mode:
                    withBlock.outfiles.append(os.path.abspath(name))
                withBlock.oldFile.__init__(self, name, mode, buffering)
            
            def __instancecheck__(self, instance):
                #print "xyz", type(instance)
                #print withBlock.oldFile
                return type(instance) == MonitoredFile or \
                        isinstance(instance, withBlock.oldFile)
        
        return MonitoredFile

