
import types
import json

# python 2.5 does not has abstract base classes, so we don't have to check
# for them
import sys
if sys.version_info >= (2, 6):
    import abc
    CLASS_TYPES = (types.ClassType, abc.ABCMeta)
else:
    CLASS_TYPES = (types.ClassType,)

from userattrs import UserValue, genPrettyFloat, genPrettyInt, genPrettyLong


def isStr(obj):
    return isinstance(obj, (str, unicode))


def handleUserAttr(obj):
    if obj.formatStr is not None:
        unit = obj.unit
        formatStr = obj.formatStr
        if isinstance(obj.value, dict):
            items = [(k, UserValue(v, unit, formatStr)) for k, v in
                    obj.value.items()]
            return dict(items)
        if isinstance(obj.value, float):
            return genPrettyFloat(obj.value, obj.formatStr)
        if isinstance(obj.value, int):
            return genPrettyInt(obj.value, obj.formatStr)
        if isinstance(obj.value, long):
            return genPrettyLong(obj.value, obj.formatStr)
        if isStr(obj):
                return obj.value
        else:
            try:
                alist = list(obj.value)
                return [UserValue(x, unit, formatStr) for x in alist]
            except:
                if isinstance(obj.value, (float, int, long, bool)):
                    return obj.value
                else:
                    return str(obj.value)
    else:
        return default(obj.value)


def default(obj):
    if isinstance(obj, UserValue):
        return handleUserAttr(obj)
    elif isStr(obj):
        return obj
    elif isinstance(obj, CLASS_TYPES):
        return [obj.__module__, obj.__name__]
    elif isinstance(obj, types.FunctionType):
        return [obj.__module__, obj.__name__]
    elif isinstance(obj, types.MethodType):
        return [obj.__module__, obj.imclass.__name__, obj.__name__]
    elif hasattr(obj, "tolist") and callable(obj.tolist):
        return [obj.tolist()]
    else:
        try:
            newList = list(obj)
            return newList
        except:
            if isinstance(obj, (float, int, long, bool)):
                return obj
            else:
                return str(obj)


def jsonStr(obj):
    try:
        json.loads(obj)
    except:
        return obj
    return json.dumps(obj)


def jsonify(obj):
    # here starts the giant hack!
    # tries to convert object into json with a lot of special cases
    if isStr(obj):
        return jsonStr(obj)
    elif isinstance(obj, UserValue) and isStr(obj.value):
        if obj.verbatim:
            return obj.value
        else:
            return jsonStr(obj.value)
    else:
        return json.dumps(obj, indent=None, default=default)

