
import shlex, subprocess

def runProcess(command, *args, **kargs):
    mergeOutputs, inputString, cwd = __parseArguments(kargs)
    splitCommand = shlex.split(command)
    realCommand = extendCommand(splitCommand, args)
    
    stdin = subprocess.PIPE
    stdout = subprocess.PIPE
    stderr = subprocess.PIPE if not mergeOutputs else subprocess.STDOUT
    p = subprocess.Popen(realCommand, stdin=stdin, stdout=stdout,
            stderr=stderr, cwd=cwd)
    (output, errout) = p.communicate(inputString)
    retval = p.wait()
    
    if (mergeOutputs):
        return (retval, output)
    else:
        return (retval, output, errout)

def extendCommand(arguments, replacements):
    restReplacements = list(replacements)
    finishedArgs = []
    for arg in arguments:
        splitarg = arg.split("%")
        numSplits = len(splitarg)
        if len(restReplacements) >= numSplits - 1:
            arg = interweave(splitarg, restReplacements[:numSplits - 1])
            del restReplacements[:numSplits - 1]
            arg = [str(x) for x in arg]
            arg = "".join(arg)
            finishedArgs.append(arg)
        else:
            errMessage("not enough additional arguments")
            raise ValueError(errMessage)
    
    if len(restReplacements) > 0:
        errMessage("too many additional arguments")
        raise ValueError(errMessage)

    return finishedArgs

def interweave(x, y):
    out = []
    i = 0
    while len(x) > i or len(y) > i:
        if len(x) > i:
            out.append(x[i])
        if len(y) > i:
            out.append(y[i])
        i += 1
    return out

def __parseArguments(dict):
    supportedArguments = ["mergeOutputs", "stdin", "cwd"]
    unsupported = list(set(dict.keys()) - set(supportedArguments))
    if len(unsupported) > 0:
        err = "keyword(s) %s are not supported by runProcess()" % \
                ", ".join(unsupported)
        raise ValueError(err)
    
    mergeOutputs = dict.get("mergeOutputs", False)
    stdin = dict.get("stdin", None)
    cwd = dict.get("cwd", None)
    return mergeOutputs, stdin, cwd

