
import vhistfile, pdfadvobjs, pdftypes, pdflayouter, layout, staticobjects, \
        resources, image, utilities

from staticobjects import INFO_ID, CATALOG_ID, PAGES_ID, PAGELABELS_ID, \
        OUTLINES_ID, RESOURCES_ID, FONT1_ID, FONT2_ID, FONT3_ID, FONT4_ID, \
        FONT5_ID, PAPERCLIP1_ID, PAPERCLIP2_ID, PAPERCLIP3_ID


class ReturnObject(object):
    def __init__(self):
        self.compressedSize = None
        self.compressedMD5 = None


def addPage(vfile, wf, staticObjs, title, page, footer, firstPage, sectionID,
        markers):
    """
    Called from addPages(): in addition to setting the text (adding it
    to a PDF stream using AddStandardLayout(), we handle outline entries
    (bookmarks). Please note that only the first page (i.e. ipage==1) will get
    a bookmark. We also add file annotations ("paperclips") where indicated by
    the <fid> markers (which have been added when the listing was generated in
    the workflow class).
    
    """
    pagesObj = staticObjs.pages
    pagelabelsObj = staticObjs.pagelabels
    outlinesObj = staticObjs.outlines
    resourcesObj = staticObjs.resources
    
    nextID = vfile.getFirstFreeID()
    contentObj = pdfadvobjs.PDFContentObject(nextID, 0)
    
    layout.addStandardLayout(contentObj, title, page, footer)
    pageObj = pdfadvobjs.PDFPageObject(nextID + 1, 0, pagesObj, resourcesObj,
            contentObj)
    vfile.addObject(contentObj)
    vfile.addObject(pageObj)
    
    nextPageNum = pagesObj.get("/Count")
    if firstPage:
        outTitle = "Section %d: %s" % (sectionID - 1, wf.title)
        outItemObj = pdfadvobjs.PDFOutlineObject(nextID+2, 0, outTitle,
                outlinesObj, pageObj)
        outlinesObj.addOutlineObject(outItemObj)
        
        vfile.addObject(outItemObj)
        pagelabelsObj.addLabel(nextPageNum, outTitle, True)
    
    for fid, pos in markers.iteritems():
        line = pos[1]
        afile = wf.files[fid]
        if afile.embedded:
            nextID = vfile.getFirstFreeID()
            annotObj = pdfadvobjs.PDFPaperclipObject(
                    nextID, 0, afile.filename.encode("utf-8"),
                    pageObj,
                    afile.embeddedPDFObjRef,
                    wf.title.encode("utf-8"),
                    line, staticobjects.PAPERCLIPS)
            pageObj.addAnnot(annotObj)
            vfile.addObject(annotObj)
    pagesObj.addPage(pageObj)


def addPages(vfile, wf, staticObjs, sectionID, pages, markers):
    """
    Iterate over all pages that have been determined by
    prepareBodyText(), which was used to break the text into
    lines and pages.
    WRONG!!!
    
    """
    numPages = len(pages)
    layouter = pdflayouter.PDFLayouter(1, 1, False)
    
    szTitle = "WS: " + wf.title
    title = layouter.genSimpleStream(szTitle)
    
    for i, (page, pageMarkers) in enumerate(zip(pages, markers)):
        isFirstpage = (i == 0)
        # i is 0-based, 1-based needed
        szFooter = "Workflow step appended: %s, page %1d/%1d"
        szFooter = szFooter % (wf.timestamp, i + 1, numPages)
        footer = layouter.genSimpleStream(szFooter)
        addPage(vfile, wf, staticObjs, title, page, footer, isFirstpage,
                sectionID, pageMarkers)


def addFile(vfile, wf, wfFile):
    """
    Gather information about one file that is either input or output
    in a particular workflow step
    
    """
    nextID = vfile.getFirstFreeID()
    fullname = wfFile.fullpath
    filename = wfFile.filename
    if wfFile.isSummary:
        content = wfFile.xmlSummary
    else:
        content = utilities.readFile(fullname)
    
    # PDFEmbeddedFileObject will write compressed size/md5 into this object
    returnobj = ReturnObject()
    
    efObj = pdfadvobjs.PDFEmbeddedFileObject(nextID, 0, filename,
            wfFile.filetype, wfFile.description, wfFile.comment, content,
            wfFile.compressionMethod, returnobj, wfFile.uncompressedMD5)
    fspecObj = pdfadvobjs.PDFFileSpecObject(nextID+1, 0, wfFile.modFilename,
            efObj, wfFile.description)
    
    vfile.addObject(efObj)
    vfile.addObject(fspecObj)
    wfFile.embeddedPDFObjRef = fspecObj
    wfFile.setCompressedProperties(returnobj.compressedSize,
            returnobj.compressedMD5)
    return fspecObj


def genOneFileProperties(wf, wfFile):
    filename = wfFile.fullpath
    if wfFile.isSummary:
        content = wfFile.xmlSummary
        wfFile.lastModified = wf.timestamp
        wfFile.genUncompressedPropertiesFromContent(content)
    else:
        # "regular files", i.e. files used for input or output during the WS
        try:
            wfFile.genUncompressedPropertiesFromFilename(filename)
        except Exception, e:
            if wfFile.embedded:
                raise


def loadStaticObjects(vfile):
    # reconstruct needed object from pdf file
    # TODO: add more checks in the fromStructObject methods
    fakePagesObj = vfile.loadObjectFromFile(PAGES_ID)
    pagesObj = pdfadvobjs.PDFPagesObject(PAGES_ID, 0)
    pagesObj.fromStructObject(fakePagesObj)
    
    fakePagelabelsObj = vfile.loadObjectFromFile(PAGELABELS_ID)
    pagelabelsObj = pdfadvobjs.PDFPageLabelsObject(PAGELABELS_ID, 0)
    pagelabelsObj.fromStructObject(fakePagelabelsObj)
    
    fakeOutlinesObj = vfile.loadObjectFromFile(OUTLINES_ID)
    id = pdfadvobjs.PDFOutlinesObject.getLastObjectID(fakeOutlinesObj)
    if id != None:
        fakeLastOutlineObj = vfile.loadObjectFromFile(id.id)
        lastOutlineObj = pdfadvobjs.PDFOutlineObject(id.id, 0, "", None, None)
        lastOutlineObj.fromStructObject(fakeLastOutlineObj)
    else:
        lastOutlineObj = None
    
    outlinesObj = pdfadvobjs.PDFOutlinesObject(OUTLINES_ID, 0)
    outlinesObj.fromStructObject(fakeOutlinesObj, lastOutlineObj)
    
    # add objects to vhistfile
    vfile.addObject(pagesObj)
    vfile.addObject(pagelabelsObj)
    vfile.addObject(outlinesObj)
    if lastOutlineObj != None:
        vfile.addObject(lastOutlineObj)
    
    objs = staticobjects.StaticObjects(info=None, catalog=None, pages=pagesObj,
            pagelabels=pagelabelsObj, outlines=outlinesObj, resources=None,
            fonts=None, paperclips=None, lastOutline=lastOutlineObj)
    
    return objs


def addPreviewImages(vfile, wf):
    images = []
    previewNum = 0
    for afile in wf.files:
        if afile.previewws:
            nextId = vfile.getFirstFreeID()
            pdfname = "/ImWS%d" % (previewNum + 1)
            previewNum += 1
            caption = afile.description
            img = image.ImageInfo(afile.fullpath, afile.filename, pdfname,
                    caption, nextId, 0)
            for obj in img.pdfObjs:
                vfile.addObject(obj)
            
            images.append(img)
    return images


def addResources(vfile, images):
    nextId = vfile.getFirstFreeID()
    resourceObj = resources.genImageResources(images, nextId, 0)
    vfile.addObject(resourceObj)
    return resourceObj


def genFileProperties(wf):
    for afile in wf.files:
        if not afile.isSummary:
            genOneFileProperties(wf, afile)


def addFiles(vfile, wf):
    # add all embedded files to the vhistfile
    for afile in wf.files:
        if afile.embedded and not afile.isSummary:
            addFile(vfile, wf, afile)


def addXmlSummary(vfile, wf):
    wf.generateXMLSummary() # result in wf.summaryfile
    genOneFileProperties(wf, wf.summaryFile)
    
    addFile(vfile, wf, wf.summaryFile)


def layoutPages(wf, images):
    summary = wf.generateTextSummary()
    summaryPages, summaryMarkers = layout.layoutSummary(summary)
    previewPages = layout.layoutPreviews(images)
    previewMarkers = [dict() for p in previewPages]
    
    pages = summaryPages + previewPages
    markers = summaryMarkers + previewMarkers
    
    return pages, markers


def appendSection(primaryOutputFile, wf):
    vfile = vhistfile.PDFFile(primaryOutputFile, wf.title)

    staticObjs = loadStaticObjects(vfile)

    images = addPreviewImages(vfile, wf)
    staticObjs.resources = addResources(vfile, images)

    genFileProperties(wf)
    addFiles(vfile, wf)
    addXmlSummary(vfile, wf)

    pages, markers = layoutPages(wf, images)
    addPages(vfile, wf, staticObjs, vfile.getNextSectionID(), pages, markers)

    vfile.appendToFile()
    del vfile
    
