"""
Module containing functions for setting and modifying critical file extensions.

Critical file extensions are extensions which are ranked by Adobe Acrobat
security critical. Examples are .ini, .exe



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: namemodifier.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

__criticalExtensions = []
def setCriticalExtensions(alist):
    """
    Sets the critical extensions list to alist.
    
    @type  alist: list of str
    @param alist: A list of critical file extensions.
    """
    global __criticalExtensions
    __criticalExtensions = alist

def modifyCriticalName(name):
    """
    Appends an underscore "_" to the given filename if the name is critical.
    
    @type  name: str or unicode
    @param name: The name of a file.
    
    @rtype:  str or unicode
    @return: The (potentially) modified filename.
    """
    global __criticalExtensions
    
    # special case
    if "." not in name:
        return name + ".noextension"
    for ext in __criticalExtensions:
        if name.endswith(ext):
            return name + "_"
    return name
