
import pdflayouter, layout, staticobjects, pdfadvobjs, vhistfile

CHARS_PER_COL = 79
LINES_PER_PAGE = 63


def titlePage(wf, titleContents, vfile, staticObjs):
    """Assembles and layouts a title page."""
    
    # shortcuts to the needed static objects
    pagesObj      = staticObjs.pages
    pagelabelsObj = staticObjs.pagelabels
    resourcesObj  = staticObjs.resources
    outlinesObj   = staticObjs.outlines
    
    # prepare title page text
    szFooter = u"File created: %s, title page" % wf.timestamp
    szTitle = u"README"
    
    pages, markers = layout.layoutSummary(titleContents, False)
    layouter = pdflayouter.PDFLayouter(CHARS_PER_COL, LINES_PER_PAGE, False)
    footer = layouter.genSimpleStream(szFooter)
    title = layouter.genSimpleStream(szTitle)
    
    # prepare loop
    nextPageNum = pagesObj.get("/Count")
    numberPages = len(pages) > 1
    firstPage = True
    
    for page in pages:
        nextID = vfile.getFirstFreeID()
        contentObj = pdfadvobjs.PDFContentObject(nextID, 0)
        layout.addStandardLayout(contentObj, title, page, footer)
        
        pageObj = pdfadvobjs.PDFPageObject(nextID + 1, 0, pagesObj, resourcesObj,
                contentObj)
        pagesObj.addPage(pageObj)
        
        if firstPage:
            outItemObj = pdfadvobjs.PDFOutlineObject(nextID + 2, 0,
                    "Titlepage", outlinesObj, pageObj)
            outlinesObj.addOutlineObject(outItemObj)
            vfile.addObject(outItemObj)
            
            pagelabelsObj.addLabel(nextPageNum, "Titlepage", numberPages)
            firstPage = False
        
        vfile.addObject(contentObj)
        vfile.addObject(pageObj)


def initStaticObjects(wf, vfile):
    """
    Creates static pdf objects, adds them to the vhist file and returns an
    object with references to these objects.
    
    """
    objs = staticobjects.genStaticObjects(wf)
    
    # add all objects to the pdf file
    vfile.addObject(objs.info)
    vfile.addObject(objs.catalog)
    vfile.addObject(objs.pages)
    vfile.addObject(objs.pagelabels)
    vfile.addObject(objs.outlines)
    vfile.addObject(objs.resources)
    for font in objs.fonts:
        vfile.addObject(font)
    for paperclip in objs.paperclips:
        vfile.addObject(paperclip)
    
    return objs


def createNewVhistFile(primaryOutputFile, wf, readme, titleContents):
    """ Adds a new titlepage section to a VHIST file."""
    vfile = vhistfile.PDFFile(primaryOutputFile, "Titlepage", readme)
    
    staticObjs = initStaticObjects(wf, vfile)
    titlePage(wf, titleContents, vfile, staticObjs)
    
    vfile.appendToFile()
    del vfile


