"""
Module containing unittests for xmlutils.py.


This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: test_xmlutils.py 2841 2013-05-23 10:27:33Z ahuesgen $
"""

'''
import xmlutils
import unittest
		
# XML string used for testing
szXML = r"""<?xml version="1.0" standalone="yes" ?>
<vhxchange>
	<workflowstep>
		<historyfile>test.pdf</historyfile>
		<desc>Quantification (HRRT)</desc>
		<tool>quant_hrrt v 0.51 of Jan 12 2005</tool>
		<timestamp>Feb 21 2006 15:17:10</timestamp>
		<infile>
			<filename>a.txt</filename>
			<desc>image to quantify (HRRT native format)</desc>
			<embed>1</embed>
		</infile>
		<infile>
			<filename>C:\SomeDirectory\SomePath\file1.i</filename>
			<desc>image to quantify (HRRT native format)</desc>
			<md5>DAD34EA5546AA95161DBFE23C5B7283D</md5>
			<filesize>4711</filesize>
			<lastmodified>Feb 17 2006 10:26:31</lastmodified>
		</infile>	
		<outfile>
			<filename>C:\Directory2\SomePath\file1_quant.i.hdr</filename>
			<desc>quantified image (HRRT native formati, header)</desc>
			<md5>77334EA5546AA95161DBFE23C5B7283D</md5>
			<filesize>5570</filesize>
			<lastmodified>Feb 21 2006 15:16:35</lastmodified>
		</outfile>		
	</workflowstep>
</vhxchange>"""


class TestXML1Parser(unittest.TestCase):
	"""A simple class for parsing a simple XML file using a DOM approach.
		usage: 
			xp = XML1Parser(myXMLString)
			# set to required level
			xp.setRootAbsolute(tag1="level1", tag2="level2")
			# return one data of one element at that level
			xp.data("myTag")
			# return list of elements
			nodes = xp.getElementList("myTag")
			# iterate over nodes
			for node in nodes: print xp.data_t(nodes, "SomeTagInSubtree")
	"""
	
	def setUp(self):
		self.xp = xmlutils.XML1Parser(szXML)
	
	def tearDown(self):	
		del self.xp

	def testSetRootAbsolute(self):
		self.assert_(self.xp.setRootAbsolute(
			tag1="workflowstep"))
		self.assert_(not self.xp.setRootAbsolute(
			tag1="quatsch", tag2="workflowstep"))
		self.assert_(self.xp.setRootAbsolute(
			tag1="workflowstep", tag2="timestamp"))

	def testData(self):
		self.assert_(self.xp.setRootAbsolute(
			tag1="workflowstep"))
		self.assert_(self.xp.data("historyfile") == "test.pdf")
		taglist =  self.xp.getRootTagList()
		reflist = ["historyfile", "desc", "tool", "timestamp", 
				"infile", "infile", "outfile" ]
		index = 0
		for tag in taglist:
			self.assert_(tag == reflist[index])
			index += 1

	def testData_t(self):
		self.assert_(self.xp.setRootAbsolute(tag1="workflowstep"))
		infiles = self.xp.getElementList("infile")
		self.assert_(len(infiles) == 2) # two "infile"-nodes in example	
		# check contents of <filename> in first infile block
		self.assert_(self.xp.data_t(infiles[0], "filename") == "a.txt")
		# check contents of <filesize> in second infile block		
		self.assert_(self.xp.data_t(infiles[1], "filesize") == "4711")

if __name__ == "__main__":
	import sys
	sys.argv.append('-v')
	unittest.main()
'''

