"""
Module containing xml file validation routines.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: xmlutils2.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

# from Python Cookbook 2nd Edition, Recipe 12.1. Checking XML Well-Formedness

from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from glob import glob
import sys

def parsefile(filename):
    """
    Checks if an xml file is wellformed or not.
    
    @type  filename: str
    @param filename: The name of the xml file, which should be checked.
    
    """
    parser = make_parser()
    parser.setContentHandler(ContentHandler())
    parser.parse(filename)

def test():
    for arg in sys.argv[1:]:
        for filename in glob(arg):
            try:
                parsefile(filename)
                print "%s is well-formed" % filename
            except Exception, e:
                print "%s is NOT well-formed! %s" % (filename, e)

# test()
